/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FBUILDER_H
#define FBUILDER_H

#include "fpm.h"
#include "wObject.h"

#include <iostream>
#include <vector>
#include <memory>
#include <Eigen/Dense>

namespace fpm
{

/**
 * @brief Aerodynamic Influence Coefficients builder
 * @authors Adrien Crovato
 * @todo add symmetry support
 * @todo implement AIC computation
 */
class FPM_API Builder : public fwk::wSharedObject
{
public:
    std::shared_ptr<Problem> pbl; ///< problem definition

    bool valid;         ///< matrices are up-to-date
    int nP;             ///< number of body panels
    int nF;             ///< number of field panels
    Eigen::MatrixXd A;  ///< body-body doublet matrix
    Eigen::MatrixXd B;  ///< body-body source matrix
    Eigen::MatrixXd C;  ///< field-body source matrix
    Eigen::MatrixXd Cx; ///< field-body x-velocity source matrix
    Eigen::MatrixXd Cy; ///< field-body y-velocity source matrix
    Eigen::MatrixXd Cz; ///< field-body z-velocity source matrix

    Builder(std::shared_ptr<Problem> _pbl);
    ~Builder() { std::cout << "~Builder()\n"; }

    void run();

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    double mu(tbox::Element *ei, tbox::Element *ej);
    double tau(tbox::Element *ei, tbox::Element *ej);
    double sigma(tbox::Element *ei, tbox::Element *ej);
    double sigmaX(tbox::Element *ei, tbox::Element *ej);
    double sigmaY(tbox::Element *ei, tbox::Element *ej);
    double sigmaZ(tbox::Element *ei, tbox::Element *ej);
};

} // namespace fpm
#endif //FBUILDER_H
