/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef FSOLVER_H
#define FSOLVER_H

#include "fpm.h"
#include "wObject.h"

#include <iostream>
#include <vector>
#include <memory>
#include <Eigen/Dense>

namespace fpm
{

/**
 * @brief Field panel solver
 * @authors Adrien Crovato
 * @todo check and implement velocity computation
 */
class FPM_API Solver : public fwk::wSharedObject
{
public:
    std::shared_ptr<Builder> aic; ///< AIC builder

    std::vector<double> phi;        ///< full potential
    std::vector<double> vPhi;       ///< perturbation potential
    std::vector<Eigen::Vector3d> U; ///< velocity
    std::vector<double> rho;        ///< density
    std::vector<double> M;          ///< mach number
    std::vector<double> Cp;         ///< pressure coefficient
    double Cl;                      ///< lift coefficient
    double Cd;                      ///< drag coefficient
    double Cs;                      ///< sideforce coefficient
    double Cm;                      ///< pitch moment coefficient (positive nose-up)

public:
    Solver(std::shared_ptr<Builder> _aic);
    ~Solver() { std::cout << "~Solver()\n"; }

    void run();
    void save(std::shared_ptr<tbox::MshExport> mshWriter, int n = 0);

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void computeFlow(const Eigen::VectorXd &mu, const Eigen::VectorXd &tau, const Eigen::VectorXd &sigma);
    void computeLoad();
};

} // namespace fpm
#endif //FSOLVER_H
